const path = require('path');
const fs = require('fs');
const Writable = require('stream').Writable;
const App = require('webwork');
const LiteEngine = require('lite');


const bindSchema = require('./validator');
const loadFileConfigMap = require('./schema')
const addSchemaDoc = require('./schema-doc');
const Console = require('console').Console;


class WWPP{

    constructor(schemaPath, webroot, options){
        // let errors = this.startError = [];
        // const errorImpl = console.error ;
        // console.error = function(){
        //
        //     errors.push.apply(errors,arguments)
        //     errorImpl.apply(this,arguments);
        // }
        // console.log(console.error+'')
        // console.error(123);

        //process.stderr.pipe(startError)
        options = options || {};
        schemaPath = path.resolve(schemaPath);
        this.config = options;
        this.schemaPath = schemaPath;
        this.webroot = webroot = webroot && path.resolve(webroot);


        if(!fs.existsSync(schemaPath)){
            //console.error('schema path is not found!!',schemaPath)
            throw new Error('schema path is not found!!',schemaPath);
        }
        ///console.log('wwpp schemaDir && templateDir',schemaDir,templateDir)

        const rootApp = new App();
        const systemApp = new App();
        {
            let engine = new LiteEngine(path.join(__dirname, '../webroot'));
            systemApp.resolveView("*.xhtml",function () {
                return engine.render.apply(engine, arguments)
            });

            addSchemaDoc(rootApp,systemApp)
        }

        if (webroot) {
            let engine = new LiteEngine(webroot);
            rootApp.resolveView('*.xhtml', function () {
                return engine.render.apply(engine, arguments)
            });
        }

        loadFileConfigMap(schemaPath)
            .then(fileConfigMap => {
                //console.log(11111)
                var rtv = bindSchema(rootApp, fileConfigMap, options);
                //console.log(22222);
                return rtv;
            },console.err).then(a=>{
                //Console.prototype.error = errorImpl;
                //setTimeout(function(){console.log('errors:',errors)},1000)

            },e=>console.error(e))
        this.rootApp = rootApp;
        this.systemApp = systemApp;
        rootApp.bind('/$',this.systemApp)
    }

    start(...ports){
        //console.log(33333)
        try{
            for(let port of ports){
                let server =  require('http').createServer(this.rootApp.receiver);
                server.setTimeout(1000*30)
                server.on('error', function (err) {
                    if (err.code === 'EADDRINUSE') { // 端口已经被使用
                        console.log('The port【'+err.port+'】 is occupied, please change other port.')
                    }
                })
                server.on('listening',function(){
                    console.log('server started：http://localhost:'+port)
                })
                server.listen(port,function(){})
            }
        }catch(e){
            console.error(e)
        }
    }
    impl(implPath,...filters){
        var schemaInstance = this;
        var rootApp = new App();
        var options = {backupApp:schemaInstance.rootApp}
        //rootApp.filters = schemaInstance.rootApp.filters.concat();
        for(var filter of filters){rootApp.intercept(filter)}
        loadFileConfigMap(implPath)
            .then(fileConfigMap => {
            //console.log(11111)
            var rtv = bindSchema(rootApp, fileConfigMap, options);
                //console.log(22222);
                return rtv;
            },console.err).then(a=>{
                //Console.prototype.error = errorImpl;
                //setTimeout(function(){console.log('errors:',errors)},1000)

            },e=>console.error(e));
        return function(req,resp){
            return rootApp.receiver(req,resp);
        }
    }

}
module.exports = WWPP;
