'use strict';

const fs = require('fs');
const async = require('asf')
const path = require('path')


const loadFileConfigMap = async(function *(dir) {
    var list = yield readDir(dir);
    let fileConfigMap = {};

    try {
        for (var n of list) {
            if (n.match(/\.js$/)) {
                let resource = require(path.join(dir, n));
                fileConfigMap['/'+n] = parseResource(resource);
            } else if (!n.match(/\./)) {
                try {
                    var list2 = yield readDir(path.join(dir, n));
                } catch (e) {
                    console.error(e)
                    continue;
                }
                for (var n2 of list2) {
                    if (n2.match(/\.js$/)) {
                        let resource = require(path.join(dir, n, n2));
                        fileConfigMap['/'+n + '/' + n2] = parseResource(resource, n);
                    }
                }
                n2 = null;//reset;
            }
        }
    } catch (e) {
        console.error('parse schema error!! file:' + (n2 || n)+e);
    }

    //console.dir(fileConfigMap)
    return fileConfigMap;
});
module.exports = loadFileConfigMap;

function readDir(dir) {
    return new Promise(function (resolve, reject) {
        fs.readdir(dir, function (err, list) {
            if (err) {
                reject(new Error('invalid schema dir:'))
            } else {
                resolve(list);
            }
        });
    });
}

function parseResource(resource, currentDirectory) {
    try{
        let path = resource.path;
        let title = resource.title;
        let methodMap = {};
        let definitions = {};
        for (var n in resource) {
            let config = resource[n];
            if (/^[A-Z]/.test(n)) {
                definitions[n] = createJSONSchema(config, currentDirectory);
            } else if (path) {
                let schema = config.schema && dep1Copy(config.schema);
                let action = config.action;
                if (schema || action) {
                    for (var type in schema) {
                        schema[type] = createJSONSchema(schema[type], currentDirectory)
                    }
                    methodMap[n] = {
                        validator: schema,
                        action: config.action
                    }
                } else {

                }
            }
        }
        return {
            path: path,
            title: title,
            definitions: definitions,
            methods: methodMap
        }
    }catch(e){
        console.error(e);
        throw e;
    }
}
function dep1Copy(obj, toObject) {
    try{

        var rtv = toObject || {};
        for (var n in obj) {
            rtv[n] = obj[n];
        }
        return rtv;
    }catch(e){
        console.error(e)
        throw e;
    }
}

function createJSONSchema(props, currentDirectory) {
    try{
        var properties = {};
        let required = [];
        for (var n in props) {
            var optional = props[n].optional
            var p = properties[n] = dep1Copy(props[n]);
            var isArray = p.type == 'array';
            if(!optional){
                required.push(n);
            }
            if(isArray) {
                p = p.items = dep1Copy(p.items);
            }
            if (p.type == 'date') {
                p.type = 'integer';
                Object.defineProperty(p, 'class', {value: 'date'});
            }
            var ref = p.$ref || p.ref;
            if (ref) {
                if(ref.charAt() == '/'){
                    //ref = ref.substr(1);
                }else{
                    ref = require('path').join((currentDirectory || '') , ref).replace(/\\/g,'/');
                }
                p.$ref = '/'+ref.replace(/#\/?/, '#/definitions/');
                //console.log( p.$ref )
                delete p.ref;
            }

            if(optional){
                p = properties[n];//reset p!!!!
                properties[n] = {
                    anyOf:[p,{type:'null'}],
                    description:p.description,
                    optional:optional
                }

            }
        }
        if (required.length == 0) {
            required = undefined
        }

        return {
            type: 'object',
            properties,
            required
        }
    }catch(e){
        console.error(e);
        throw e;
    }
}
