'use strict';

const fs = require('fs');
const path = require('path');
const genApiDoc = require('./schema-doc');
const LiteEngine = require('lite');
const Webwork = require('webwork')
module.exports = buildSystemIntercaptor;
function buildSystemIntercaptor(rootApp,webroot) {

    app.get('/api/',function *(req,resp){
        try{
            resp.writeHead(404, {'Content-Type': 'text/html;charset=utf-8'})
            this.routes = rootApp._routes[1].map(routerConfig);
            return "/404.xhtml"
        }catch(e){
            return 404;
        }
    })
    function routerConfig(router){
        let title = matchBinder(rootApp, router.path).config.title;
        let path = router.path;
        return {title,path}
    }

    app.get('/api/*', function *(req,resp){
        //console.log("!!!!!!!",req.params[0])
        try{
            let path = decodeURIComponent(req.params[0]);
            //console.log(path)
            let binder = matchBinder(rootApp, path)
            let model = binder && genApiDoc(rootApp, binder, path);
            if(model){
                for(var n in model){
                    this[n] = model[n];
                }
                return '/api-doc.xhtml';
            }else{
                //console.log('model:',binder,path)
                resp.writeHead(404, {'Content-Type': 'text/html;charset=utf-8'})
                this.api = path;
                this.routes = rootApp._routes[1].map(routerConfig);
                return "/404.xhtml"
            }
        }catch(e){
            //console.log(e);
            return 404;
        }
    })
    return app;
}

function  doUpload(req,resp,webroot,fileBasePath){
    var multiparty = require('multiparty');
    var util = require('util')
    var path = require('path');
    var form = new multiparty.Form();
    var fs = require('fs');
    var dir = webroot + '/upload/'+fileBasePath;
    if(fs.existsSync(dir) && fs.statSync(dir).isDirectory()){
        //console.log(111111)
        return new Promise(function (resolve,reject){
            form.parse(req, function(err, fields, files) {
                resp.writeHead(200, {'content-type': 'text/plain'});
                //resp.write('received upload:\n\n');
                //resp.end(util.inspect({err,fields: fields, files: files}));
                var fileList = [];
                for(var file in files){
                    //console.log(n,files[n])
                    let i =1;
                    let targetFile = file;
                    var target = path.join(dir,targetFile);
                    //while(fs.existsSync(target)){
                    //    target = path.join(dir,targetFile = file.replace(/(\.\w+)?$/,'[' + ++i +']$1'));
                    //}
                    //console.log(files[file])
                    fs.renameSync(files[file][0].path,target);
                    fileList.push({name:file,path:targetFile});
                }
                var rtv = JSON.stringify({success:true,list:fileList});
                //fs.writeFileSync(require('path').join(dir,'index.json'),rtv)
                resp.end(rtv)
                resolve(null);
            })
        });
    }else{
        return Promise.reject({error:'dir file not exists:'+dir});
    }

}

//function  buildFileInterceptor(app,webroot){}
function matchBinder(app, path) {
    let list = app._routes[1];
    let len = list.length;
    let startIndex = 0;
    while (startIndex < len) {
        let binder = list[startIndex++];
        if (path == binder.path) {
            return binder;
        }
    }
}
//var markdown = require('markdown').markdown;


function markdownAndHighlight(text) {
    //var html = markdown.toHTML( text);
    //console.log(marked('I am using __markdown__.'));
    // Synchronous highlighting with highlight.js
    var marked = require('marked');
    var hljs = require('highlight.js');
    marked.setOptions({
        highlight: function (code, lan) {
            //console.log("@@@@@@",lan)
            return require('highlight.js').highlightAuto(code,lan &&  [lan]).value;
        }
    });
    var renderer = new marked.Renderer();
    var options = {renderer};
    var oldCode = renderer.code;
    renderer.code = function(code,lan){
        switch(lan){
            case 'qrcode':
                return "<img src='"+qrcodeToDataURL(code)+"'/>";

            default:
                if(code.match(/^\s*(?:graph|gantt|sequenceDiagram)/)){
                    return '<div class="mermaid">'+code+'</div>';
                }
                return oldCode.apply(this,arguments);
        }
    }

    var html = marked(text,options);
    return html;
}
function qrcodeToDataURL(text){
    var rtv ;
    require('qrcode').toDataURL(text,function(err,code){
        rtv = code;
    })
    return rtv;
}
function print404(app, req, resp,webroot) {
    //console.log('new 404!!')
    var file = webroot && path.join(webroot,req.url);
    var validRes = req.url.match(/^[\/\w\- ]+\.(?:(js|css)|(jpge?|png|gif)|(xhtml|md))$/);
    var fstat;
    if(validRes && file && (fstat = fs.existsSync(file)&&fs.statSync(file))){
        var crypto = require('crypto');
        var md5= '"'+crypto.createHash('md5').update(fstat.size+'/'+fstat.mtime.getTime()).digest('base64')+'"';
        var oldMd5 = req.headers['if-none-match'];
        var contentType = validRes[1] && "text/"+(
                    validRes[1] == 'js'?'javascript':validRes[1]
                )+';charset=utf8'
                || validRes[2] && "image/"+validRes[2]
                || "text/html;charset=utf8"

        var headers = {
            "Access-Control-Allow-Origin":"*",
            "Content-Type": contentType,
            "ETag":md5};
        if(md5 == oldMd5){
            resp.writeHead(304, headers);
            //console.log('304 response: '+filepath)
            resp.end();
        }else{
            if(validRes[1]){
                resp.writeHead(200,headers)
            }else if(validRes[2]){
                resp.writeHead(200,headers);
            }else if(validRes[3] == 'xhtml'){
                return req.url
            }else if(validRes[3] == 'md'){
                var text = fs.readFileSync(file,'utf-8');
                this.content =markdownAndHighlight(text);
                //    path = this.config.root .resolve('.'+path.path).path;
                //text = "<html><body>"++"</body></html>";
                return "/md-layout.xhtml"
            }
            return new Promise(function(resolve){fs.createReadStream(file).pipe(resp).on('close',resolve)})
        }
    }else if(req.url && req.url != '/'){
        resp.writeHead(404, {'Content-Type': 'text/html;charset=utf-8'})
        let body = `${req.url == '/$/api/'? '':'Request Resource Not Found:${req.url}<br>'}
        Available Routers List:<hr>
        <ul>
        ${app._routes[0].join('\n').replace(/.+/g, (a, i) => {
            //app.routes[1][i].config.title//
            let title = matchBinder(app, a).config.title;
            return '<li><a href="/$/api/' + encodeURIComponent(a) + '">' + a + '</a>' + (title ? ' \t(' + title + i + ')' : '') + '</li>'
        })}
        </ul>`;
        this.content = body;
        this.routes = app.routes;
        return "/404.xhtml"
    //resp.end(body);
    }else if(webroot){
        var file = './readme.md'
        if(!fs.existsSync(file)){
            file = webroot && path.join(webroot,'../readme.md');
        }
        var text = fs.readFileSync(file,'utf-8');
        this.content =markdownAndHighlight(text);
        //    path = this.config.root .resolve('.'+path.path).path;
        //text = "<html><body>"++"</body></html>";
        return "/md-layout.xhtml"

    }
}

