'use strict';

const Ajv = require('ajv')
const async = require('asf')
module.exports = bindSchema;

function * validatorInterceptor(req, resp, next, config) {//validate
    let validator = config.validator;
    if (validator && validator.request) {
        var value = yield req.value;
        if (!validator.request(value)) {//value changed
            let error = 'input validate error:' + validator.request.errors.map(ajvErrorMap);
            console.error(error);
            error = new Error(error);
            error.code = -1;
            throw error;
        }
    }
    var model = this;
    var result = yield next(req, resp);

    if (validator && validator.response) {
        var errors = async(function *() {
            for (var key in model) {
                var value =  yield model[key];
                if(value != null){
                    model[key] =value;
                }else{
                    delete model[key];
                }
            }
            if (!validator.response(model)) {
                return validator.response.errors
            }
        })().then(function (errors) {
            if (errors) {
                let error = 'output validate error:' + errors.map(ajvErrorMap);
                error = new Error(error);
                error.code = 1;
                throw error;
            }
        });


        if (result) {
            errors.catch(error => console.error(error)) ;
        } else {
            yield errors;
        }
    }
    return result;

}
function ajvErrorMap(err) {
    return err.dataPath + ':\t' + err.message || err;
}
function findBackupConfig(backupApp, path) {
    //    this.routes = [[],[],{}];
    if (backupApp) {
        let routes = backupApp.routes;
        let routeIndex = routes[0].indexOf(String(path));
        if (routeIndex >= 0) {
            return app.routes[1][routeIndex].config;
        }
    }
}
function bindSchema(app, fileConfigMap, options) {
    try {


        let inputAjv = new Ajv({coerceTypes: true});
        let outputAjv = new Ajv();
        let schemaMap = {};
        let backupApp = options && options.backup;
        for (var fileName in fileConfigMap) {
            let id = fileName.replace(/\.js$/, '');

            let schema = {id: id, definitions: fileConfigMap[fileName].definitions};
            for (let fileDefinition in schema.definitions) {
                schema.definitions[fileDefinition].id = id + '#/definitions/' + fileDefinition;
                //console.log(fileDefinition,schema.definitions[fileDefinition].id)
            }
            //console.log('schema id:'+id)
            try {
                inputAjv.addSchema(schema, id);
                outputAjv.addSchema(schema, id);
            } catch (e) {
                console.error('add schema define error:', id, e);
            }
            schemaMap[id] = schema
        }

        for (var fileName in fileConfigMap) {
            let fileConfig = fileConfigMap[fileName];
            let title = fileConfig.title;
            let path = fileConfig.path;
            if (path != null) {
                let ns = fileName.replace(/[^\/\\]+$/, '');
                if (ns) {
                    path = ns.slice(0, -1) + path
                }
                let configMap = {};
                let actionMap = {};
                let methodMap = fileConfig.methods;
                let backupConfig = findBackupConfig(backupApp, path) || {};

                try {
                    for (var m in methodMap) {
                        let action = methodMap[m].action;
                        let validator = methodMap[m].validator;
                        if (action || validator) {
                            if (!backupConfig[m] && validator) {//backupConfig 验证不可覆盖
                                //var validator.request

                                try {////response coerceTypes true;
                                    var requestValidator = validator.request && inputAjv.compile(validator.request);
                                } catch (e) {
                                    console.error('add request validator schema（'+fileName+'） error:', validator.request, e);
                                    continue;
                                }
                                try {//response coerceTypes false;
                                    var responseValidator = validator.response && outputAjv.compile(validator.response);
                                } catch (e) {
                                    console.error('add response validator schema（'+fileName+'） error:', validator.response, e);
                                    continue;
                                }
                                configMap[m] = {
                                    validator: {
                                        request: requestValidator,
                                        response: responseValidator,
                                        ref: closureRefs(fileName, schemaMap, validator.request, validator.response)
                                    },
                                    modelHandler: responseValidator && createResponseProxyHander(validator.response, outputAjv)
                                };
                            } else {
                                configMap[m] = backupConfig[m]
                            }
                            actionMap[m] = action
                        }
                    }
                } catch (e) {
                    console.error('add router schema error:', e);
                }

                configMap.path = path;
                configMap.title = title;
                app.bind(configMap, actionMap)
            }
        }
        app.intercept(validatorInterceptor);
    }catch(e){
        console.error(e);
        throw e;
    }
}
// function clearNull(v){
//     console.log('clearNull!!!')
//     if(typeof v == 'object'){
//         for(var n in v){
//             if(v[n] == null){
//                 delete v[n];
//             }else{
//                 clearNull( v[n]);
//             }
//         }
//     }
// }
function createResponseProxyHander(schema, ajv) {
    try{
        var props = schema.properties;
        var vm = {};
        for (var n in props) {
            vm[n] = ajv.compile(props[n]);
        }
        //console.log('createResponseProxyHander!set',schema)
        return props && {
            set: function (target, key, value, receiver) {
                //console.log('createResponseProxyHander!set')
                //value && clearNull(value)
                if (props.hasOwnProperty(key)) {
                    var validate = vm[key];
                    if (value!=null && !(value instanceof Promise ) && !validate(value)) {
                        try {
                            var schema = JSON.stringify(validate.schema);
                            var json = JSON.stringify(value);
                        } catch (e) {
                            json = e;
                        }

                        var error = new Error('valid property ' + key + '!!\n'
                            + 'json data: \n' + json + '\n\nnot match schema:\n' + schema + '\n\n'
                            + 'validate error:\n' + (validate.errors.map(e => key + ajvErrorMap(e))) + '\n\n'
                        );

                        error.code = 1;
                        throw error;
                    }
                } else {
                    let error = new Error('property is not defined!' + key);
                    error.code = 1;
                    throw error;
                }
                return Reflect.set(target, key, value, receiver);
            }
        }
    }catch(e){
        console.error(e);
        throw e;
    }
}
function closureRefs(where , refMap, ...vs){
    try{
        var map = {};
        var refs = [];
        for (let v of vs) {
            if (!v || !v.properties) {
                continue;
            }
            let props = v.properties;
            for (var n in props) {
                try {
                    var p = props[n];
                    var ref = p.$ref || p.items && p.items.$ref;
                    if (ref && refs.indexOf(ref) < 0) {
                        refs.push(ref);
                        let [fileName, defKey] = ref.split('#');
                        if(!refMap[fileName]){
                            console.error(`ref fileName not found! ${fileName} @ ${where}`)
                            continue;
                        }
                        map[ref] = refMap[fileName].definitions[defKey.replace('/definitions/', '')];
                        vs.push(map[ref])
                        //console.log('###########',ref)
                    }
                }catch(e){
                    console.error(e);
                }
            }
        }
        return map;
    }catch(e){
        console.error(e);
        throw e;
    }
}
